"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createDeployFiles = createDeployFiles;
const path_1 = __importDefault(require("path"));
const url_1 = __importDefault(require("url"));
const normalize_path_1 = __importDefault(require("normalize-path"));
const pick_1 = __importDefault(require("ramda/src/pick"));
const DEPENDENCIES_FIELD = ['dependencies', 'devDependencies', 'optionalDependencies'];
const INHERITED_MANIFEST_KEYS = [
    'name',
    'description',
    'version',
    'private',
    'author',
    'bin',
    'scripts',
    'packageManager',
    'dependenciesMeta',
    'peerDependenciesMeta',
];
function createDeployFiles({ allProjects, deployDir, lockfile, lockfileDir, manifest, projectId, rootProjectManifestDir, }) {
    const deployedProjectRealPath = path_1.default.resolve(lockfileDir, projectId);
    const inputSnapshot = lockfile.importers[projectId];
    const targetSnapshot = {
        ...inputSnapshot,
        specifiers: {},
        dependencies: {},
        devDependencies: {},
        optionalDependencies: {},
    };
    const targetPackageSnapshots = {};
    for (const name in lockfile.packages) {
        const inputDepPath = name;
        const inputSnapshot = lockfile.packages[inputDepPath];
        const resolveResult = resolveLinkOrFile(inputDepPath, {
            lockfileDir,
            projectRootDirRealPath: rootProjectManifestDir,
        });
        const outputDepPath = resolveResult
            ? createFileUrlDepPath(resolveResult, allProjects)
            : inputDepPath;
        targetPackageSnapshots[outputDepPath] = convertPackageSnapshot(inputSnapshot, {
            allProjects,
            deployDir,
            deployedProjectRealPath,
            lockfileDir,
            projectRootDirRealPath: rootProjectManifestDir,
        });
    }
    for (const importerPath in lockfile.importers) {
        if (importerPath === projectId)
            continue;
        const projectSnapshot = lockfile.importers[importerPath];
        const projectRootDirRealPath = path_1.default.resolve(lockfileDir, importerPath);
        const packageSnapshot = convertProjectSnapshotToPackageSnapshot(projectSnapshot, {
            allProjects,
            deployDir,
            lockfileDir,
            deployedProjectRealPath,
            projectRootDirRealPath,
        });
        const depPath = createFileUrlDepPath({ resolvedPath: projectRootDirRealPath }, allProjects);
        targetPackageSnapshots[depPath] = packageSnapshot;
    }
    for (const field of DEPENDENCIES_FIELD) {
        const targetDependencies = targetSnapshot[field] ?? {};
        const targetSpecifiers = targetSnapshot.specifiers;
        const inputDependencies = inputSnapshot[field] ?? {};
        for (const name in inputDependencies) {
            const spec = inputDependencies[name];
            const resolveResult = resolveLinkOrFile(spec, {
                lockfileDir,
                projectRootDirRealPath: path_1.default.resolve(lockfileDir, projectId),
            });
            if (!resolveResult) {
                targetSpecifiers[name] = targetDependencies[name] = spec;
                continue;
            }
            targetSpecifiers[name] = targetDependencies[name] =
                resolveResult.resolvedPath === deployedProjectRealPath ? 'link:.' : createFileUrlDepPath(resolveResult, allProjects);
        }
    }
    const result = {
        lockfile: {
            ...lockfile,
            overrides: undefined, // the effects of package overrides should already be part of the package snapshots
            patchedDependencies: undefined,
            packageExtensionsChecksum: undefined, // the effects of the package extensions should already be part of the package snapshots
            pnpmfileChecksum: undefined, // the effects of the pnpmfile should already be part of the package snapshots
            importers: {
                ['.']: targetSnapshot,
            },
            packages: targetPackageSnapshots,
        },
        manifest: {
            ...(0, pick_1.default)(INHERITED_MANIFEST_KEYS, manifest),
            dependencies: targetSnapshot.dependencies,
            devDependencies: targetSnapshot.devDependencies,
            optionalDependencies: targetSnapshot.optionalDependencies,
            pnpm: {
                ...manifest.pnpm,
                overrides: undefined, // the effects of package overrides should already be part of the package snapshots
                patchedDependencies: undefined,
                packageExtensions: undefined, // the effects of the package extensions should already be part of the package snapshots
            },
        },
    };
    if (lockfile.patchedDependencies) {
        result.lockfile.patchedDependencies = {};
        result.manifest.pnpm.patchedDependencies = {};
        for (const name in lockfile.patchedDependencies) {
            const patchInfo = lockfile.patchedDependencies[name];
            const resolvedPath = path_1.default.resolve(rootProjectManifestDir, patchInfo.path);
            const relativePath = (0, normalize_path_1.default)(path_1.default.relative(deployDir, resolvedPath));
            result.manifest.pnpm.patchedDependencies[name] = relativePath;
            result.lockfile.patchedDependencies[name] = {
                hash: patchInfo.hash,
                path: relativePath,
            };
        }
    }
    return result;
}
function convertPackageSnapshot(inputSnapshot, opts) {
    const inputResolution = inputSnapshot.resolution;
    let outputResolution;
    if ('integrity' in inputResolution) {
        outputResolution = inputResolution;
    }
    else if ('tarball' in inputResolution) {
        outputResolution = { ...inputResolution };
        if (inputResolution.tarball.startsWith('file:')) {
            const inputPath = inputResolution.tarball.slice('file:'.length);
            const resolvedPath = path_1.default.resolve(opts.lockfileDir, inputPath);
            const outputPath = (0, normalize_path_1.default)(path_1.default.relative(opts.deployDir, resolvedPath));
            outputResolution.tarball = `file:${outputPath}`;
            if (inputResolution.path)
                outputResolution.path = outputPath;
        }
    }
    else if (inputResolution.type === 'directory') {
        const resolvedPath = path_1.default.resolve(opts.lockfileDir, inputResolution.directory);
        const directory = (0, normalize_path_1.default)(path_1.default.relative(opts.deployDir, resolvedPath));
        outputResolution = { ...inputResolution, directory };
    }
    else if (inputResolution.type === 'git') {
        outputResolution = inputResolution;
    }
    else {
        const resolution = inputResolution; // `never` is the type guard to force fixing this code when adding new type of resolution
        throw new Error(`Unknown resolution type: ${JSON.stringify(resolution)}`);
    }
    return {
        ...inputSnapshot,
        resolution: outputResolution,
        dependencies: convertResolvedDependencies(inputSnapshot.dependencies, opts),
        optionalDependencies: convertResolvedDependencies(inputSnapshot.optionalDependencies, opts),
    };
}
function convertProjectSnapshotToPackageSnapshot(projectSnapshot, opts) {
    const resolution = {
        type: 'directory',
        directory: (0, normalize_path_1.default)(path_1.default.relative(opts.deployDir, opts.projectRootDirRealPath)),
    };
    const dependencies = convertResolvedDependencies(projectSnapshot.dependencies, opts);
    const optionalDependencies = convertResolvedDependencies(projectSnapshot.optionalDependencies, opts);
    return {
        dependencies,
        optionalDependencies,
        resolution,
    };
}
function convertResolvedDependencies(input, opts) {
    if (!input)
        return undefined;
    const output = {};
    for (const key in input) {
        const spec = input[key];
        const resolveResult = resolveLinkOrFile(spec, opts);
        if (!resolveResult) {
            output[key] = spec;
            continue;
        }
        if (resolveResult.resolvedPath === opts.deployedProjectRealPath) {
            output[key] = 'link:.'; // the path is relative to the lockfile dir, which means '.' would reference the deploy dir
            continue;
        }
        output[key] = createFileUrlDepPath(resolveResult, opts.allProjects);
    }
    return output;
}
function resolveLinkOrFile(spec, opts) {
    // try parsing `spec` as `spec(peers)`
    const hasPeers = /^(?<spec>[^()]+)(?<peers>\(.+\))$/.exec(spec);
    if (hasPeers) {
        const result = resolveLinkOrFile(hasPeers.groups.spec, opts);
        if (!result)
            return undefined;
        if (result.suffix) {
            throw new Error(`Something goes wrong, suffix is not undefined: ${result.suffix}`);
        }
        result.suffix = hasPeers.groups.peers;
        return result;
    }
    // try parsing `spec` as either @scope/name@pref or name@pref
    const renamed = /^@(?<scope>[^@]+)\/(?<name>[^@]+)@(?<pref>.+)$/.exec(spec) ?? /^(?<name>[^@]+)@(?<pref>.+)$/.exec(spec);
    if (renamed)
        return resolveLinkOrFile(renamed.groups.pref, opts);
    const { lockfileDir, projectRootDirRealPath } = opts;
    if (spec.startsWith('link:')) {
        const targetPath = spec.slice('link:'.length);
        return {
            scheme: 'link:',
            resolvedPath: path_1.default.resolve(projectRootDirRealPath, targetPath),
        };
    }
    if (spec.startsWith('file:')) {
        const targetPath = spec.slice('file:'.length);
        return {
            scheme: 'file:',
            resolvedPath: path_1.default.resolve(lockfileDir, targetPath),
        };
    }
    return undefined;
}
function createFileUrlDepPath({ resolvedPath, suffix }, allProjects) {
    const depFileUrl = url_1.default.pathToFileURL(resolvedPath).toString();
    const project = allProjects.find(project => project.rootDirRealPath === resolvedPath);
    const name = project?.manifest.name ?? path_1.default.basename(resolvedPath);
    return `${name}@${depFileUrl}${suffix ?? ''}`;
}
//# sourceMappingURL=createDeployFiles.js.map